NOISE_LEVELS = [0, .1, .2, .3, .4, .5, .6, .7, .8, .9]

NIGEP_AVAILABLE_KWARGS = [
    'execution_name',
    'model',
    'batch_size',
    'input_shape',
    'x_data',
    'y_data',
    'target_names',
    'class_mode',
    'k_fold_n',
    'epochs',
    'callbacks',
    'noise_levels',
    'save_models',
    'evaluate_models',
]